/**
 * Input parameters saved in a map to avoid mess in HTML.
 * Map is used to map parameter by its ID to its properties:
 * Properties:
 * > name - parameter name (with HTML tags, if applicable)
 * > unit - parameter unit (with HTML tags, if applicable)
 * > type - integer or float
 * > min - minimum allowed input value
 * > value - parameter default value
 * > max - maximum allowed input value
 * > description - description about the parameter (with HTML tags, if applicable) (currently not used)
 * > step - input parameters step (while input box is open use arrow keys UP and DOWN to increase/decrease input value)
 */

const PARAM_MAP = new Map();

/* INPUT PARAMETERS FROM HERE */

PARAM_MAP.set('vsmin', {
	name: 'V<sub>S(min)</sub>',
	unit: 'V',
	type: 'float',
	min: 5.5,
	value: 7,
	max: 15.5,
	description: 'Minimum required battery voltage for operation',
	step: 0.1
});

PARAM_MAP.set('vsmax', {
	name: 'V<sub>S(max)</sub>',
	unit: 'V',
	type: 'float',
	min: 7,
	value: 15.5,
	max: 40,
	description: 'Maximum applied battery voltage for continues operation. Note: reverse diode not considered in calc',
	step: 0.1
});

PARAM_MAP.set('vout', {
	name: 'V<sub>OUT</sub>',
	unit: 'V',
	type: 'float',
	min: 1.4,
	value: 6,
	max: 14.75,
	description: 'Typical output voltage at LED operating point',
	step: 0.1
});

PARAM_MAP.set('iout', {
	name: 'I<sub>OUT</sub>',
	unit: 'mA',
	type: 'float',
	min: 15,
	value: 200,
	max: 360,
	description: 'Output current',
	step: 5
});

PARAM_MAP.set('tambient', {
	name: 'T<sub>A</sub>',
	unit: '°C',
	type: 'float',
	min: -40,
	value: 85,
	max: 150,
	description: 'Ambient temperature',
	step: 1
});

PARAM_MAP.set('rthja', {
	name: 'R<sub>thJA</sub>',
	unit: 'K/W',
	type: 'float',
	min: 10,
	value: 65,
	max: 100,
	description: 'Thermal resistance of TLD1114-1EP observed in application condition',
	step: 1
});

PARAM_MAP.set('v_overhead', {
	name: 'V<sub>OVERHEAD</sub>',
	unit: 'V',
	type: 'float',
	min: 0,
	value: 5,
	max: 15,
	description: 'At V<sub>S(max)</sub> + V<sub>OVERHEAD</sub> the power shift current limitation will start',
	step: 0.1
});

PARAM_MAP.set('rdson', {
	name: 'R<sub>DSON</sub>',
	unit: 'Ω',
	type: 'float',
	min: 0.00,
	value: 1,
	max: 100,
	description: 'Typical R<sub>DSON</sub> at operating point',
	step: 0.1
});

PARAM_MAP.set('vgstyp', {
	name: 'V<sub>GS(typ)</sub>',
	unit: 'V',
	type: 'float',
	min: 1,
	value: 2,
	max: 5,
	description: 'Typical V<sub>GS</sub> at operating point of NMOS',
	step: 0.1
});

PARAM_MAP.set('r1_selected', {
	name: 'R<sub>1</sub>',
	unit: 'Ω',
	type: 'float',
	min: 0,
	value: 20.00,
	max: 1000,
	description: 'Recommended value for Resistor between V<sub>S</sub> and Power Shift NMOS Drain connector',
	step: 0
});

PARAM_MAP.set('r2_selected', {
	name: 'R<sub>2</sub>',
	unit: 'Ω',
	type: 'float',
	min: 0,
	value: 8.10,
	max: 1000,
	description: 'Recommended value for Resistor between Power Shift NMOS source connector and PWR_SHS pin',
	step: 0
});

/* INPUT PARAMETERS TO HERE */

/* OUTPUT PARAMETERS FROM HERE */

PARAM_MAP.set('pvmax', {
	name: 'P<sub>V(max)</sub>',
	unit: 'W',
	type: 'float',
	min: 0,
	value: 1,
	max: 1000,
	description: 'Maximum allowed power dissipation of the LED driver (Calculated using R<sub>th</sub>, T<sub>A</sub> & T<sub>J(max)</sub>)',
	step: 0
});

PARAM_MAP.set('rset', {
	name: 'R<sub>SET</sub>',
	unit: 'kΩ',
	type: 'float',
	min: 0,
	value: 3.02,
	max: 10000,
	description: 'Recommended value for R<sub>SET</sub> resistor',
	step: 0
});

PARAM_MAP.set('r1', {
	name: 'R<sub>1</sub>',
	unit: 'Ω',
	type: 'float',
	min: 0,
	value: 20.68,
	max: 1000,
	description: 'Recommended value for Resistor between V<sub>S</sub> and Power Shift NMOS Drain connector',
	step: 0
});

PARAM_MAP.set('r2', {
	name: 'R<sub>2</sub>',
	unit: 'Ω',
	type: 'float',
	min: 0,
	value: 7.14,
	max: 1000,
	description: 'Recommended value for Resistor between Power Shift NMOS source connector and PWR_SHS pin',
	step: 0
});

/* OUTPUT PARAMETERS TO HERE */

/* PRODUCT PARAMETERS FROM HERE */

PARAM_MAP.set('vin_set', {
	name: 'V<sub>IN_SET</sub>',
	unit: 'V',
	type: 'float',
	min: 0,
	value: 1.22,
	max: 1000,
	description: 'Reference voltage for output current reference generation',
	step: 0
});

PARAM_MAP.set('k1', {
	name: 'k<sub>1</sub>',
	unit: '',
	type: 'float',
	min: 0,
	value: 900,
	max: 9999,
	description: 'I<sub>OUTL</sub>/I<sub>IN_SET</sub>',
	step: 0
});

PARAM_MAP.set('k2', {
	name: 'k<sub>2</sub>',
	unit: '',
	type: 'float',
	min: 0,
	value: 890,
	max: 9999,
	description: 'I<sub>OUTH</sub>/I<sub>IN_SET</sub>',
	step: 0
});

PARAM_MAP.set('ipsmax', {
	name: 'I<sub>PS(max)</sub>',
	unit: 'mA',
	type: 'float',
	min: 0,
	value: 500,
	max: 10000,
	description: 'Maximum allowed current through power shift circuit',
	step: 0
});

PARAM_MAP.set('vgs_pwr_sh', {
	name: 'V<sub>GS_PWR_SH</sub>',
	unit: 'V',
	type: 'float',
	min: 0,
	value: 5.50,
	max: 1000,
	description: 'Typical power shift gate source clamping voltage',
	step: 0
});

PARAM_MAP.set('rshunt', {
	name: 'R<sub>SHUNT</sub>',
	unit: 'Ω',
	type: 'float',
	min: 0,
	value: 1.50,
	max: 1000,
	description: 'Value of internal shunt resistor',
	step: 0
});

PARAM_MAP.set('vsmin_spec', {
	name: 'V<sub>S(min)SPEC</sub>',
	unit: 'V',
	type: 'float',
	min: 0,
	value: 5.50,
	max: 1000,
	description: 'Minimum supply voltage',
	step: 0
});

PARAM_MAP.set('vsmax_spec', {
	name: 'V<sub>S(max)SPEC</sub>',
	unit: 'V',
	type: 'float',
	min: 0,
	value: 40,
	max: 1000,
	description: 'Minimum supply voltage',
	step: 0
});

PARAM_MAP.set('iout_min', {
	name: 'I<sub>OUT(min)</sub>',
	unit: 'mA',
	type: 'float',
	min: 0,
	value: 13.5,
	max: 1000,
	description: 'Minimum output current',
	step: 0
});

PARAM_MAP.set('iout_max', {
	name: 'I<sub>OUT(max)</sub>',
	unit: 'mA',
	type: 'float',
	min: 0,
	value: 360,
	max: 1000,
	description: 'Maximum usable output current',
	step: 0
});

PARAM_MAP.set('vps_cc', {
	name: 'V<sub>PS_CC</sub>',
	unit: 'V',
	type: 'float',
	min: 0,
	value: 0.75,
	max: 1000,
	description: 'Minimum voltage drop for current control',
	step: 0
});

PARAM_MAP.set('vout_cc', {
	name: 'V<sub>OUT_CC</sub>',
	unit: 'V',
	type: 'float',
	min: 0,
	value: 1.40,
	max: 1000,
	description: 'Minimum output voltage for current control',
	step: 0
});

PARAM_MAP.set('tjmax', {
	name: 'T<sub>J(max)</sub>',
	unit: 'UNITUNITUNIT',
	type: 'float',
	min: 0,
	value: 150.00,
	max: 1000,
	description: 'Maximum operating junction temperature',
	step: 0
});

PARAM_MAP.set('IDIDIDIDIDID', {
	name: 'NAMENAMENAME',
	unit: 'UNITUNITUNIT',
	type: 'float',
	min: 0000000000000,
	value: 0000000000000,
	max: 0000000000000,
	description: 'DESCRIPTION',
	step: 0000000000000
});

/* PRODUCT PARAMETERS TO HERE */

/**
 * Make a deep copy of PARAM_MAP so that PARAM_MAP stays the same all the time (to be able to retrieve default values later).
 */
function cloneParamMap(parameters_map) {
	let mapCopy = new Map();

	parameters_map.forEach(function(value, key) {
		mapCopy.set(key, JSON.parse(JSON.stringify(value)));
	});

	return mapCopy;
}