let pseudo_param_map = new Map();

const svg_output_ids = '#pvmax, #rset, #r1, #r2';

const pseudocode_table_header = ['n', 'V<sub>S<sub>n</sub></sub>', 'I<sub>PS<sub>n</sub></sub>', 'I<sub>OUTS<sub>n</sub></sub>', 'I<sub>OUT<sub>n</sub></sub>', 'P<sub>TLD1114<sub>n</sub></sub>', 'P<sub>R<sub>1<sub>n</sub></sub></sub>', 'P<sub>R<sub>2<sub>n</sub></sub></sub>'];

let PSEUDOCODE_TABLE;

/**
 * Set value of specified result parameter after it has been updated/calculated and before calling the updateFormulas(param_map) function.
 */
function updatePseudoParamValue(param_id, new_value) {

    // If it is a number then round it to NDEC decimal places. Otherwise do nothing.
    new_value = Number.isFinite(new_value) ? roundDecimals(new_value, NDEC) : new_value;

    if (typeof(pseudo_param_map.get(param_id)) != 'undefined' && pseudo_param_map.get(param_id) != null) {
        pseudo_param_map.get(param_id).value = new_value;
    } else {
        pseudo_param_map.set(param_id, {value: new_value});
    }
}

/**
 * Set value of specified pseudocode table result parameter on table row hover. Does not update pseudo_param_map entry. Updates only one parameter at a time.
 */
function updatePseudoParamValueImmediately(param_id, new_value) {

    let isNumber = Number.isFinite(new_value);

    // If it is a number then round it to NDEC decimal places. Otherwise do nothing.
    new_value = isNumber ? roundDecimals(new_value, NDEC) : new_value;

    document.querySelectorAll('[id="math_' + param_id + '"]').forEach(function(element) {
        element.innerHTML = new_value;
        element.dispatchEvent(new Event("change"));

         // If number is negative add "contains-negative-number" class to its classList, otherwise remove it.
         if (isNumber ? new_value < 0 : false) {
            element.parentNode.classList.add('contains-negative-number');
        } else {
            element.parentNode.classList.remove('contains-negative-number');
        }
    });
}

/**
 * JavaScript function which sets the updated value in the pseudocode formulas.
 */
function updateFormulas() {

    // Select all elements from the HTML DOM which have IDs matching the pattern 'math_*'.
    let entries = document.querySelectorAll('[id^="math_"]');

    entries.forEach(function(element) {

        let element_id = element.id.substring("math_".length);

        // If the element exists in the pseudo_param_map then update its value
        if (pseudo_param_map.get(element_id) !== undefined) {
            let new_value = pseudo_param_map.get(element_id).value;
            let isNumber = Number.isFinite(new_value);

            element.innerHTML = new_value;
            element.dispatchEvent(new Event("change"));

            // If number is negative add "contains-negative-number" class to its classList, otherwise remove it.
            if (isNumber ? new_value < 0 : false) {
                element.parentNode.classList.add('contains-negative-number');
            } else {
                element.parentNode.classList.remove('contains-negative-number');
            }
        }
    });

    // Also update the voltage interpolation table.
    updatePseudoCodeTable();
}

/**
 * Add tooltips to elements from the SVG object and the HTML DOM. The function must/should be called from the HTML DOM each time svg loads or reloads. 
 */
function onSVGFullyLoaded(svg_document) {
    
    /* Add tooltips to elements from the SVG object. */
    {
        let output_elements = Array.from(svg_document.contentDocument.querySelectorAll(svg_output_ids));

        output_elements.forEach(function(el) {

            let fPopper = document.getElementById('formula-popper-' + el.id);

            // In case that element does not exist.
            if(typeof(fPopper) == 'undefined' || fPopper == null) {
                return;
            }

            el.parentNode.parentNode.addEventListener("mouseover", function() {
                let param_position = el.getBoundingClientRect();
                let schem_position = svg_document.getBoundingClientRect();

                let posX = Math.round(schem_position.left + param_position.right + 20 + window.scrollX);
                let posY = Math.round(schem_position.top + param_position.top + window.scrollY);

                $(fPopper).css({"transform":"translate3d(" + posX + "px, " + posY + "px, 0px)"});
                $(fPopper).show();
            });

            el.parentNode.parentNode.addEventListener("mouseout", function() {
                $(fPopper).hide();
            });
        });
    }
}

/**
 * Creates a table using data generated in calculation.js by function generatePowerData().
 */
function createPseudoCodeTable(pseudocode_table) {
    
    pseudocode_table.innerHTML = ''; // Clear the content.

    var tblBody = document.createElement("tbody");
    
    for (var j = 0; j < 22; j++) {
        var row = document.createElement("tr");

        for (var i = 0; i < pseudocode_table_header.length; i++) {
            var cell = document.createElement("td");

            if (i == 0) {
                cell.innerHTML =  j > 0 ? (j - 1) : pseudocode_table_header[i];
            } else if (j == 0) {
                cell.innerHTML = pseudocode_table_header[i];
            } else {
                cell.innerHTML =  'null';
            }
            
            row.appendChild(cell);
        }

        // Add event listener on table row hover.
        if (j > 0) {
            let row_id = j - 1;
            row.addEventListener("mouseover", function(){ onPseudoCodeTableRowHover("" + row_id + "")} );
        }

        tblBody.appendChild(row);
    }
    
    pseudocode_table.appendChild(tblBody);
    pseudocode_table.setAttribute("border", "1");

    PSEUDOCODE_TABLE = pseudocode_table;
}

/**
 * Update the pseudocode table (voltage interpolation table)
 */
function updatePseudoCodeTable() {

    if (typeof(PSEUDOCODE_TABLE) == 'undefined' || PSEUDOCODE_TABLE == null) {
        return;
    }

    if (typeof(powerData) == 'undefined' || powerData == null) {
        return;
    }

    for (let i = 0; i < powerData.length; i++) {
        for (let j = 0; j < powerData[0].length - 1; j++) {

            // The array can still contain negative values (which makes no sense for the data we have here).
            let current_value = powerData[i][j] > 0 ? powerData[i][j] : 0;

            let unit;
            switch (pseudocode_table_header[j + 1].charAt(0)) {
                case 'P': unit = "W"; break;
                case 'V': unit = "V"; break;
                case 'I': unit = "mA"; break;
                default: unit = ''; break;
            }

            if (unit == 'mA') {
                PSEUDOCODE_TABLE.rows[i + 1].cells[j + 1].innerHTML = roundDecimals(current_value * 1000, NDEC) + ' ' + unit;
            } else {
                PSEUDOCODE_TABLE.rows[i + 1].cells[j + 1].innerHTML = roundDecimals(current_value, NDEC) + ' ' + unit;
            }
        }

        // If I_OUT(max) is greater than the defined I_OUT then highlight those cells as erroneus.
        if (roundDecimals(powerData[i][3], NDEC + 1) > iout) {
            PSEUDOCODE_TABLE.rows[i + 1].cells[4].classList.add("error-result");
        } else {
            PSEUDOCODE_TABLE.rows[i + 1].cells[4].classList.remove("error-result");
        }

        // Highlight P_TLD1114 cell with max value.
        if (i == p1114max_index) {
            PSEUDOCODE_TABLE.rows[i + 1].cells[5].classList.add("max-ptld");
        } else {
            PSEUDOCODE_TABLE.rows[i + 1].cells[5].classList.remove("max-ptld");
        }

        // Highlight P_R_1 cell with max value.
        if (i == pr1_max_index) {
            PSEUDOCODE_TABLE.rows[i + 1].cells[6].classList.add("max-pr1");
        } else {
            PSEUDOCODE_TABLE.rows[i + 1].cells[6].classList.remove("max-pr1");
        }

        // Highlight P_R_2 cell with max value.
        if (i == pr2_max_index) {
            PSEUDOCODE_TABLE.rows[i + 1].cells[7].classList.add("max-pr2");
        } else {
            PSEUDOCODE_TABLE.rows[i + 1].cells[7].classList.remove("max-pr2");
        }
    }

}

/**
 * On hover over a single row of the pseudocode table (voltage interpolation table): update dynamic formulas which can be found under the table.
 * Those formulas show how we get the results you see in that specific row in the table.
 */
function onPseudoCodeTableRowHover(row_id) {
    updatePseudoParamValueImmediately('n', row_id);
    updatePseudoParamValueImmediately('vs_n', powerData[row_id][0]);
    updatePseudoParamValueImmediately('ips_n_si', powerData[row_id][1]);
    updatePseudoParamValueImmediately('ips_n', powerData[row_id][1] * 1000);
    updatePseudoParamValueImmediately('iouts_n', powerData[row_id][2]* 1000);
    updatePseudoParamValueImmediately('iout_n_si', powerData[row_id][3]);
    updatePseudoParamValueImmediately('iout_n', powerData[row_id][3]* 1000);
    updatePseudoParamValueImmediately('ptld_n', powerData[row_id][4]);
    updatePseudoParamValueImmediately('pr1_n', powerData[row_id][5]);
    updatePseudoParamValueImmediately('pr2_n', powerData[row_id][6]);
    updatePseudoParamValueImmediately('ips_case', toolMOSFETVersion == NO_MOSFET ? 0 : powerData[row_id][0] - vout < vgstyp ? 1 : powerData[row_id][0] - vout < vgs_pwr_sh ? 2 : 3);
}
